<?php
/**
 * Plugin Name: Parks Custom Post Type
 * Description: A simple plugin which adds a custom post type for Parks.
 * Version: 1.0
 * Author: Rustamveer Singh
 * Author URI: https://bcaur.com/
 * Text Domain: com-parks-plugin
 */

if (!defined('ABSPATH')) {
    exit;
}

class CoMParksCustomPostType
{
    public function __construct()
    {
        // Register hooks
        add_action('init', [$this, 'register_post_type']);
        add_action('init', [$this, 'register_taxonomy']);
        add_action('add_meta_boxes', [$this, 'add_meta_boxes']);
        add_action('save_post', [$this, 'save_meta_box_data']);
        add_action('wp_enqueue_scripts', [$this, 'enqueue_styles_scripts']);
        add_action('after_setup_theme', [$this, 'setup_theme_compatibility']);

        // Register shortcode
        add_shortcode('park_list', [$this, 'parks_list_shortcode']);

        // WP Ajax Filter hooks
        add_action('wp_ajax_filter_parks', [$this, 'ajax_filter_parks']);
        add_action('wp_ajax_nopriv_filter_parks', [$this, 'ajax_filter_parks']);
    }

    /**
     * Registers the 'Park' custom post type.
     * Sets up labels, supports, menu icon, and REST API visibility.
     */
    public function register_post_type()
    {
        $labels = [
            'name' => __('Parks', 'com-parks-plugin'),
            'singular_name' => __('Park', 'com-parks-plugin'),
            'menu_name' => __('Parks', 'com-parks-plugin'),
            'name_admin_bar' => __('Park', 'com-parks-plugin'),
            'add_new' => __('Add New', 'com-parks-plugin'),
            'add_new_item' => __('Add New Park', 'com-parks-plugin'),
            'edit_item' => __('Edit Park', 'com-parks-plugin'),
            'view_item' => __('View Park', 'com-parks-plugin'),
            'all_items' => __('All Parks', 'com-parks-plugin'),
            'search_items' => __('Search Parks', 'com-parks-plugin'),
            'not_found' => __('No parks found.', 'com-parks-plugin'),
            'not_found_in_trash' => __('No parks found in trash.', 'com-parks-plugin'),
        ];

        $args = [
            'labels' => $labels,
            'public' => true,
            'has_archive' => true,
            'supports' => ['title', 'editor', 'custom-fields'],
            'menu_icon' => 'dashicons-palmtree', // from here menu icon can be changed
            'show_in_rest' => true,
            'rewrite' => ['slug' => 'parks'],
        ];

        register_post_type('park', $args);
    }

    /**
     * Registers the 'Facility' taxonomy for the Park custom post type.
     * The taxonomy will allow parks to be categorized by different facilities.
     */
    public function register_taxonomy()
    {
        $labels = [
            'name' => __('Facilities', 'com-parks-plugin'),
            'singular_name' => __('Facility', 'com-parks-plugin'),
            'search_items' => __('Search Facilities', 'com-parks-plugin'),
            'all_items' => __('All Facilities', 'com-parks-plugin'),
            'edit_item' => __('Edit Facility', 'com-parks-plugin'),
            'update_item' => __('Update Facility', 'com-parks-plugin'),
            'add_new_item' => __('Add New Facility', 'com-parks-plugin'),
            'new_item_name' => __('New Facility Name', 'com-parks-plugin'),
            'menu_name' => __('Facilities', 'com-parks-plugin'),
        ];

        $args = [
            'labels' => $labels,
            'hierarchical' => true,
            'show_ui' => true,
            'show_in_rest' => true,
            'rewrite' => ['slug' => 'facilities'],
        ];

        register_taxonomy('facility', 'park', $args);
    }

    public function add_meta_boxes()
    {
        add_meta_box(
            'parks_details_meta_box',
            __('Park Details', 'com-parks-plugin'),
            [$this, 'render_meta_box'],
            'park',
            'normal',
            'high'
        );
    }

    public function render_meta_box($post)
    {
        wp_nonce_field('parks_save_meta_box_data', 'parks_meta_box_nonce');

        $location = get_post_meta($post->ID, '_park_location', true);
        $hours_weekday = get_post_meta($post->ID, '_park_hours_weekday', true);
        $hours_weekend = get_post_meta($post->ID, '_park_hours_weekend', true);
        $short_description = get_post_meta($post->ID, '_park_short_description', true);

        ?>
        <p>
            <label for="park_location"><strong><?php esc_html_e('Location:', 'com-parks-plugin'); ?></strong></label>
            <input type="text" class="widefat" id="park_location" name="park_location" value="<?php echo esc_attr($location); ?>" />
        </p>
        <p>
            <label for="park_hours_weekday"><strong><?php esc_html_e('Weekday Hours:', 'com-parks-plugin'); ?></strong></label>
            <input type="text" class="widefat" id="park_hours_weekday" name="park_hours_weekday"
                value="<?php echo esc_attr($hours_weekday); ?>" />
        </p>
        <p>
            <label for="park_hours_weekend"><strong><?php esc_html_e('Weekend Hours:', 'com-parks-plugin'); ?></strong></label>
            <input type="text" class="widefat" id="park_hours_weekend" name="park_hours_weekend"
                value="<?php echo esc_attr($hours_weekend); ?>" />
        </p>
        <p>
            <label
                for="park_short_description"><strong><?php esc_html_e('Short Description:', 'com-parks-plugin'); ?></strong></label>
            <textarea class="widefat" id="park_short_description" name="park_short_description"
                rows="3"><?php echo esc_textarea($short_description); ?></textarea>
        </p>
        <?php
    }

    public function save_meta_box_data($post_id)
    {
        if (!isset($_POST['parks_meta_box_nonce']) || !wp_verify_nonce($_POST['parks_meta_box_nonce'], 'parks_save_meta_box_data')) {
            return;
        }

        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }

        if (!current_user_can('edit_post', $post_id)) {
            return;
        }

        if (isset($_POST['park_location'])) {
            update_post_meta($post_id, '_park_location', sanitize_text_field($_POST['park_location']));
        }
        if (isset($_POST['park_hours_weekday'])) {
            update_post_meta($post_id, '_park_hours_weekday', sanitize_text_field($_POST['park_hours_weekday']));
        }
        if (isset($_POST['park_hours_weekend'])) {
            update_post_meta($post_id, '_park_hours_weekend', sanitize_text_field($_POST['park_hours_weekend']));
        }
        if (isset($_POST['park_short_description'])) {
            update_post_meta($post_id, '_park_short_description', sanitize_textarea_field($_POST['park_short_description']));
        }
    }

    /**
     * Renders the shortcode for listing parks with a filter by facility.
     * Generates a dropdown for facilities and an empty container to load parks via AJAX.
     */
    public function parks_list_shortcode()
    {
        ob_start();

        ?>
        <div class="filter">
            <form id="filter-form">
                <label for="facility">Filter by Facility:</label>
                <select name="facility" id="facility">
                    <option value="">Select Facility</option>
                    <?php
                    $facilities = get_terms([
                        'taxonomy' => 'facility',
                        'orderby' => 'name',
                        'hide_empty' => false,
                    ]);

                    if (!empty($facilities) && !is_wp_error($facilities)) {
                        foreach ($facilities as $facility) {
                            echo '<option value="' . esc_attr($facility->slug) . '">' . esc_html($facility->name) . '</option>';
                        }
                    }
                    ?>
                </select>
            </form>
        </div>

        <div class="parks-list"></div>
        <?php

        return ob_get_clean();
    }

    /**
     * enqueue styles & scripts
     * @return void
     */
    public function enqueue_styles_scripts()
    {
        wp_enqueue_style('parks-styles', plugin_dir_url(__FILE__) . 'assets/css/parks-styles.css');
        wp_enqueue_script('parks-ajax', plugin_dir_url(__FILE__) . 'assets/js/parks-ajax.js', ['jquery'], null, true);
        wp_localize_script('parks-ajax', 'parksAjax', [
            'ajaxUrl' => admin_url('admin-ajax.php'),
        ]);
    }

    /**
     * Sets up theme compatibility based on whether the site is using a block-based theme or a traditional PHP based theme.
     * - If using a block theme, it adds a filter to modify the post content and include custom fields (location, hours, etc.).
     * - If using a traditional theme, it loads a custom template for single park pages.
     * @return void
     */
    public function setup_theme_compatibility()
    {
        if (wp_is_block_theme()) {
            add_filter('the_content', [$this, 'add_custom_fields_to_content']);
        } else {
            add_filter('template_include', [$this, 'load_single_template']);
        }
    }

    /**
     * If using a block theme, it adds a filter to modify the post content and include custom fields (location, hours, etc.)
     * @param mixed $content
     * @return mixed
     */
    public function add_custom_fields_to_content($content)
    {
        if ('park' === get_post_type() && is_singular('park')) {
            $location = get_post_meta(get_the_ID(), '_park_location', true);
            $hours_weekday = get_post_meta(get_the_ID(), '_park_hours_weekday', true);
            $hours_weekend = get_post_meta(get_the_ID(), '_park_hours_weekend', true);
            $short_description = get_post_meta(get_the_ID(), '_park_short_description', true);

            $extra_content = '<div class="park-details">';
            $extra_content .= '<p><strong>Location:</strong> ' . esc_html($location) . '</p>';
            $extra_content .= '<p><strong>Weekday Hours:</strong> ' . esc_html($hours_weekday) . '</p>';
            $extra_content .= '<p><strong>Weekend Hours:</strong> ' . esc_html($hours_weekend) . '</p>';
            $extra_content .= '<p>' . esc_html($short_description) . '</p>';
            $extra_content .= '</div>';

            return $extra_content . $content;
        }

        return $content;
    }

    /**
     * Loads the custom template for single park pages.
     * Checks if the plugin's custom template exists, and uses it for displaying park details.
     */
    public function load_single_template($template)
    {
        if (is_singular('park')) {
            $plugin_template = plugin_dir_path(__FILE__) . 'templates/single-park.php';
            if (file_exists($plugin_template)) {
                return $plugin_template;
            }
        }

        return $template;
    }

    /**
     * Handles the AJAX request to filter parks based on the selected facility.
     * Retrieves the filtered parks and outputs them as HTML for the frontend.
     */
    public function ajax_filter_parks()
    {
        $facility_filter = isset($_POST['facility']) ? sanitize_text_field($_POST['facility']) : '';

        $args = [
            'post_type' => 'park',
            'posts_per_page' => -1,
            'tax_query' => [],
        ];

        if ($facility_filter) {
            $args['tax_query'][] = [
                'taxonomy' => 'facility',
                'field' => 'slug',
                'terms' => $facility_filter,
                'operator' => 'IN',
            ];
        }

        $query = new WP_Query($args);

        if ($query->have_posts()) {
            while ($query->have_posts()) {
                $query->the_post();
                $location = get_post_meta(get_the_ID(), '_park_location', true);
                $hours_weekday = get_post_meta(get_the_ID(), '_park_hours_weekday', true);
                $short_description = get_post_meta(get_the_ID(), '_park_short_description', true);
                $permalink = get_permalink();

                echo '<div class="park-item">';
                echo '<a href="' . esc_url($permalink) . '">';
                echo '<h2>' . esc_html(get_the_title()) . '</h2>';
                echo '<p><strong>Location:</strong> ' . esc_html($location) . '</p>';
                echo '<p><strong>Weekday Hours:</strong> ' . esc_html($hours_weekday) . '</p>';
                echo '<p>' . esc_html(wp_trim_words($short_description, 20)) . '</p>';
                echo '</a>';
                echo '</div>';
            }
        } else {
            echo '<p>No parks found.</p>';
        }

        wp_die(); // this is required to terminate properly.
    }

}

new CoMParksCustomPostType();
